import React, {Component} from 'react';
import SortTitle from './sort-table-utils/SortTitle';

export default function sortTable() {
    return function (WrappedComponent) {
        class WithSubscription extends Component {
            static displayName = `WithSubscription(${WrappedComponent.displayName || WrappedComponent.name || 'Component'})`;

            componentWillUnmount() {
                // 当前组件卸载，释放各个service占用的资源
                Object.keys(this.services).forEach(item => {
                    const se = this.services[item];
                    if (se.release) {
                        se.release();
                    }
                });
            }

            render() {
                const {columns, onSort} = this.props;
                const sortColumns = columns && columns.map(item => {
                    let {title, isSort, dataIndex} = item;

                    this.sortOrder = this.sortDataIndex === dataIndex ? this.sortOrder : false;

                    const sortTitle = (
                        <SortTitle
                            sortOrder={this.sortOrder}
                            onClick={(so) => {
                                this.sortOrder = so;
                                this.sortDataIndex = dataIndex;
                                onSort(so, dataIndex);
                            }}
                        >{title}</SortTitle>
                    );

                    return isSort ? {...item, title: sortTitle} : item;
                });
                return <WrappedComponent {...this.props} columns={sortColumns}/>;
            }
        }

        return WithSubscription;
    };
}
